/*
 * Temperature Sensor Receiver
 *
 * Temperature sensor receiver using a 2400 bps
 * 433.92 MHz ASK receiver.
 *
 * File:   temperature_receiver.ino
 * Author: Ryan Clarke
 *
 */

#include <SoftwareSerial.h>


// Address of transmitter
#define TX_ADDRESS 0x11


// Software serial port for receiver
// RX: pin 10
// TX: pin 11
SoftwareSerial rfSerial(10, 11);

/*
 * Setup function
 */
void setup()
{
  // Serial-to-USB @ 9600 bps
  // Serial RF receiver @ 2400 bps
  Serial.begin(9600);
  rfSerial.begin(2400);
}

void loop()
{
  unsigned char address, low_byte, high_byte, checksum;
  unsigned char checksum_result;
  unsigned short data;
  float voltage, temperature_c, temperature_f;
  
  // Capture a data frame (address, high byte, low byte, checksum)
  while(rfSerial.available() < 4);
  address = rfSerial.read();
  high_byte = rfSerial.read();
  low_byte = rfSerial.read();
  checksum = rfSerial.read();
  
  // Check the data vs the checksum by adding them.
  checksum_result = address + high_byte + low_byte + checksum;
  
  // The result of the checksum addition should be 0.
  if(checksum_result == 0)
  {
    // Verify that the packet is from the correct transmitter
    if(address == TX_ADDRESS)
    {
      // Combine the bytes into one word.
      data = (high_byte << 8) | low_byte;
      
      // Compute the voltage read from the transmitter ADC
      voltage = (data / 1024.0) * 2.048;
      
      // Convert to a temperature. The TMP36 measures 25 deg C at 750 mV
      // and has a temperature gradient of 10 mV / 1 deg C.
      temperature_c = ((voltage - 0.750) * 100) + 25;
      temperature_f = (1.8 * temperature_c) + 32;
      
      Serial.print("Voltage: ");
      Serial.println(voltage, 3);
      Serial.print("Temperature (F): ");
      Serial.println(temperature_f, 1);
      Serial.println();
    }
  }
}

